@@ -637,7 +637,7 @@ int __stdcall myatoi(char *s)
   return (int)v;
 }
 
-BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ul_reason_for_call, LPVOID lpReserved) {
+extern "C" BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ul_reason_for_call, LPVOID lpReserved) {
   g_hInstance=hInst;
   switch (ul_reason_for_call) {
     case DLL_PROCESS_ATTACH:
