@@ -156,6 +156,9 @@ struct FieldType {
 
   int    nParentIdx;  // this is used to store original windowproc for LINK
   HANDLE hImage; // this is used by image/icon field to save the handle to the image
+
+  int    nField; // field number in INI file
+  char  *pszHwndEntry; // "HWND" or "HWND2"
 };
 
 // initial buffer size.  buffers will grow as required.
@@ -482,6 +485,9 @@ int WINAPI ReadSettings(void) {
     };
     FieldType *pField = pFields + nIdx;
 
+    pField->nField = nCtrlIdx + 1;
+    pField->pszHwndEntry = "HWND";
+
     wsprintf(szField, "Field %d", nCtrlIdx + 1);
     pszAppName = szField;
 
@@ -565,6 +571,8 @@ int WINAPI ReadSettings(void) {
       pNewField->rect.bottom = pField->rect.bottom;
       pNewField->rect.top    = pField->rect.top;
       pField->rect.right = pNewField->rect.left - 3;
+      pNewField->nField = nCtrlIdx + 1;
+      pNewField->pszHwndEntry = "HWND2";
       nNumFields++;
       nIdx++;
     }
@@ -602,8 +610,11 @@ LRESULT WINAPI WMCommandProc(HWND hWnd, UINT id, HWND hwndCtl, UINT codeNotify)
           GetWindowText(pField->hwnd, szBrowsePath, sizeof(szBrowsePath));
 
         tryagain:
+          GetCurrentDirectory(BUFFER_SIZE, szResult); // save working dir
           if ((pField->nFlags & FLAG_SAVEAS) ? GetSaveFileName(&ofn) : GetOpenFileName(&ofn)) {
             mySetWindowText(pField->hwnd, szBrowsePath);
+            SetCurrentDirectory(szResult); // restore working dir
+                                           // OFN_NOCHANGEDIR doesn't always work (see MSDN)
             break;
           }
           else if (szBrowsePath[0] && CommDlgExtendedError() == FNERR_INVALIDFILENAME) {
@@ -798,7 +809,7 @@ BOOL CALLBACK cfgDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 #endif
 
 // pFields[nIdx].nParentIdx is used to store original windowproc
-int WINAPI StaticLINKWindowProc(HWND hWin, UINT uMsg, LPARAM wParam, WPARAM lParam)
+int WINAPI StaticLINKWindowProc(HWND hWin, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   int StaticField = FindControlIdx(GetDlgCtrlID(hWin));
   if (StaticField < 0)
@@ -925,6 +936,7 @@ int WINAPI createCfgDlg()
   BOOL fFocused = FALSE;
 
 #define DEFAULT_STYLES (WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS)
+#define RTL_EX_STYLES (WS_EX_RTLREADING | WS_EX_LEFTSCROLLBAR)
 
   for (int nIdx = 0; nIdx < nNumFields; nIdx++) {
     static struct {
@@ -938,72 +950,72 @@ int WINAPI createCfgDlg()
         DEFAULT_STYLES,
         DEFAULT_STYLES | SS_RIGHT,
         WS_EX_TRANSPARENT,
-        WS_EX_TRANSPARENT | WS_EX_RTLREADING },
+        WS_EX_TRANSPARENT | RTL_EX_STYLES },
       { "STATIC",       // FIELD_ICON
         DEFAULT_STYLES | SS_ICON,
         DEFAULT_STYLES | SS_ICON,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "STATIC",       // FIELD_BITMAP
         DEFAULT_STYLES | SS_BITMAP,
         DEFAULT_STYLES | SS_BITMAP,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "BUTTON",       // FIELD_BROWSEBUTTON
         DEFAULT_STYLES | WS_TABSTOP,
         DEFAULT_STYLES | WS_TABSTOP,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "BUTTON",       // FIELD_LINK
         DEFAULT_STYLES | WS_TABSTOP | BS_OWNERDRAW,
         DEFAULT_STYLES | WS_TABSTOP | BS_OWNERDRAW | BS_RIGHT,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "BUTTON",       // FIELD_BUTTON
         DEFAULT_STYLES | WS_TABSTOP,
         DEFAULT_STYLES | WS_TABSTOP,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "BUTTON",       // FIELD_GROUPBOX
         DEFAULT_STYLES | BS_GROUPBOX,
         DEFAULT_STYLES | BS_GROUPBOX | BS_RIGHT,
         WS_EX_TRANSPARENT,
-        WS_EX_TRANSPARENT | WS_EX_RTLREADING },
+        WS_EX_TRANSPARENT | RTL_EX_STYLES },
       { "BUTTON",       // FIELD_CHECKBOX
         DEFAULT_STYLES | WS_TABSTOP | BS_TEXT | BS_VCENTER | BS_AUTOCHECKBOX | BS_MULTILINE,
         DEFAULT_STYLES | WS_TABSTOP | BS_TEXT | BS_VCENTER | BS_AUTOCHECKBOX | BS_MULTILINE | BS_RIGHT | BS_LEFTTEXT,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "BUTTON",       // FIELD_RADIOBUTTON
         DEFAULT_STYLES | WS_TABSTOP | BS_TEXT | BS_VCENTER | BS_AUTORADIOBUTTON | BS_MULTILINE,
         DEFAULT_STYLES | WS_TABSTOP | BS_TEXT | BS_VCENTER | BS_AUTORADIOBUTTON | BS_MULTILINE | BS_RIGHT | BS_LEFTTEXT,
         0,
-        WS_EX_RTLREADING },
+        RTL_EX_STYLES },
       { "EDIT",         // FIELD_TEXT
         DEFAULT_STYLES | WS_TABSTOP | ES_AUTOHSCROLL,
         DEFAULT_STYLES | WS_TABSTOP | ES_AUTOHSCROLL | ES_RIGHT,
         WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE,
-        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RTLREADING },
+        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | RTL_EX_STYLES },
       { "EDIT",         // FIELD_FILEREQUEST
         DEFAULT_STYLES | WS_TABSTOP | ES_AUTOHSCROLL,
         DEFAULT_STYLES | WS_TABSTOP | ES_AUTOHSCROLL | ES_RIGHT,
         WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE,
-        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RTLREADING },
+        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | RTL_EX_STYLES },
       { "EDIT",         // FIELD_DIRREQUEST
         DEFAULT_STYLES | WS_TABSTOP | ES_AUTOHSCROLL,
         DEFAULT_STYLES | WS_TABSTOP | ES_AUTOHSCROLL | ES_RIGHT,
         WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE,
-        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RTLREADING },
+        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | RTL_EX_STYLES },
       { "COMBOBOX",     // FIELD_COMBOBOX
         DEFAULT_STYLES | WS_TABSTOP | WS_VSCROLL | WS_CLIPCHILDREN | CBS_AUTOHSCROLL | CBS_HASSTRINGS,
         DEFAULT_STYLES | WS_TABSTOP | WS_VSCROLL | WS_CLIPCHILDREN | CBS_AUTOHSCROLL | CBS_HASSTRINGS,
         WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE,
-        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RIGHT | WS_EX_RTLREADING },
+        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RIGHT | RTL_EX_STYLES },
       { "LISTBOX",      // FIELD_LISTBOX
         DEFAULT_STYLES | WS_TABSTOP | WS_VSCROLL | LBS_DISABLENOSCROLL | LBS_HASSTRINGS | LBS_NOINTEGRALHEIGHT,
         DEFAULT_STYLES | WS_TABSTOP | WS_VSCROLL | LBS_DISABLENOSCROLL | LBS_HASSTRINGS | LBS_NOINTEGRALHEIGHT,
         WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE,
-        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RIGHT | WS_EX_RTLREADING }
+        WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE | WS_EX_RIGHT | RTL_EX_STYLES }
     };
 
     FieldType *pField = pFields + nIdx;
@@ -1110,6 +1122,14 @@ int WINAPI createCfgDlg()
       NULL
     );
 
+    {
+      char szField[64];
+      char szHwnd[64];
+      wsprintf(szField, "Field %d", pField->nField);
+      wsprintf(szHwnd, "%d", hwCtrl);
+      WritePrivateProfileString(szField, pField->pszHwndEntry, szHwnd, pszFilename);
+    }
+
     if (hwCtrl) {
       // Sets the font of IO window to be the same as the main window
       mySendMessage(hwCtrl, WM_SETFONT, (WPARAM)hFont, TRUE);
@@ -1342,7 +1362,7 @@ void WINAPI showCfgDlg()
 
   while (!g_done) {
     MSG msg;
-    int nResult = GetMessage(&msg, NULL, 0, 0);
+    GetMessage(&msg, NULL, 0, 0);
     if (!IsDialogMessage(hConfigWindow,&msg) && !IsDialogMessage(hMainWindow,&msg))
     {
       TranslateMessage(&msg);
