@@ -19,9 +19,9 @@
   3. This notice may not be removed or altered from any source distribution.
 
 */
-#include <windows.h>
-#include "resource.h"
+
 #include "makensisw.h"
+#include "resource.h"
 #include "toolbar.h"
 #include "noclib.h"
 
@@ -162,6 +162,7 @@ void ErrorMessage(HWND hwnd,const char *str) {
 // Altered by Darren Owen (DrO) on 1/10/2003
 void Items(HWND hwnd, int on){
   UINT mf = (!on ? MF_GRAYED : MF_ENABLED);
+  UINT nmf = (!on ? MF_ENABLED : MF_GRAYED);
 
   if(!on)
       g_sdata.focused_hwnd = GetFocus();
@@ -189,6 +190,7 @@ void Items(HWND hwnd, int on){
   EnableMenuItem(g_sdata.menu,IDM_BROWSESCR,mf);
   EnableMenuItem(g_sdata.menu,IDM_RECOMPILE_TEST,mf);
   EnableMenuItem(g_sdata.menu,IDM_COMPRESSOR,mf);
+  EnableMenuItem(g_sdata.menu,IDM_CANCEL,nmf);
 
   EnableToolBarButton(IDM_SAVE,on);
   // Altered by Darren Owen (DrO) on 6/10/2003
@@ -918,3 +920,17 @@ BOOL FileExists(char *fname)
     return true;
   }
 }
+
+HMENU FindSubMenu(HMENU hMenu, UINT uId)
+{
+  MENUITEMINFO mii = {
+    sizeof(MENUITEMINFO),
+    MIIM_SUBMENU,
+  };
+
+  mii.hSubMenu = NULL;
+
+  GetMenuItemInfo(hMenu, uId, FALSE, &mii);
+
+  return mii.hSubMenu;
+}
