@@ -190,7 +190,7 @@ static LRESULT CALLBACK ParentWndProc(HWND hwnd, UINT message, WPARAM wParam, LP
   return 0;
 }
 
-BOOL APIENTRY DllMain(HINSTANCE _hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
+extern "C" BOOL APIENTRY DllMain(HINSTANCE _hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
 {
   hModule = _hModule;
   return TRUE;
@@ -230,9 +230,11 @@ __declspec(dllexport) void download (HWND   parent,
   char buf[1024];
   char url[1024];
   char filename[1024];
+  static char proxy[1024];
   BOOL bSuccess=FALSE;
   int timeout_ms=30000;
   int getieproxy=1;
+  int manualproxy=0;
 
   char *error=NULL;
 
@@ -274,6 +276,12 @@ __declspec(dllexport) void download (HWND   parent,
     timeout_ms=my_atoi(url+9);
     popstring(url);
   }
+  if (!lstrcmpi(url, "/PROXY")) {
+    getieproxy=0;
+    manualproxy=1;
+    popstring(proxy);
+    popstring(url);
+  }
   if (!lstrcmpi(url, "/NOIEPROXY")) {
     getieproxy=0;
     popstring(url);
@@ -340,6 +348,9 @@ __declspec(dllexport) void download (HWND   parent,
         buf[8192-1]=0;
         RegCloseKey(hKey);
       }
+      if (manualproxy == 1) {
+        p = proxy;
+      }
 
       DWORD start_time=GetTickCount();
       get=new JNL_HTTPGet(JNL_CONNECTION_AUTODNS,16384,(p&&p[0])?p:NULL);
@@ -369,15 +380,13 @@ __declspec(dllexport) void download (HWND   parent,
           break;
         }
 
-        Sleep(25);
-
         st = get->run ();
 
         if (st == -1) {
           lstrcpyn(url, get->geterrorstr(), sizeof(url));
           error = url;
         } else if (st == 1) {
-          if (sofar < cl)
+          if (sofar < cl || get->get_status () != 2)
             error="download incomplete";
           else
           {
@@ -412,7 +421,9 @@ __declspec(dllexport) void download (HWND   parent,
               }
             }
 
+            int data_downloaded = 0;
             while ((len = get->bytes_available ()) > 0) {
+              data_downloaded++;
               if (len > 8192)
                 len = 8192;
               len = get->get_bytes (buf, len);
@@ -455,6 +466,8 @@ __declspec(dllexport) void download (HWND   parent,
             }
             if (GetTickCount() > last_recv_time+timeout_ms)
               error = "Downloading timed out.";
+            else if (!data_downloaded)
+              Sleep(10);
 
           } else {
             error = "Bad response status.";
