@@ -1,9 +1,19 @@
 #include <windows.h>
 #include <stdio.h>
+#include <ctype.h>
 #include <commctrl.h>
 
 /*
 
+version 0.35
+* drag & drop support
+
+version 0.34
+* preserve zip timestamps
+
+version 0.33
+* Added solid compression checkbox
+
 version 0.32
 * Fixed codepage problems
 
@@ -36,6 +46,7 @@ HANDLE g_hThread;
 char g_cmdline[1024];
 int g_extracting;
 int g_compressor;
+int g_compressor_solid;
 int g_mui;
 int g_zipfile_size;
 
@@ -167,7 +178,6 @@ int tempzip_make(HWND hwndDlg, char *fn)
   g_extracting=1;
   do {
     char filename[MAX_PATH];
-    WCHAR filenameW[MAX_PATH];
     unz_file_info info;
 
     unzGetCurrentFileInfo(f,&info,filename,sizeof(filename),NULL,0,NULL,0);
@@ -175,11 +185,7 @@ int tempzip_make(HWND hwndDlg, char *fn)
     // was zip created on MS-DOS/Windows?
     if ((info.version & 0xFF00) == 0)
     {
-      // convert from OEM codepage to ANSI codepage
-      if (MultiByteToWideChar(CP_OEMCP, 0, filename, -1, filenameW, MAX_PATH))
-      {
-        WideCharToMultiByte(CP_ACP, 0, filenameW, -1, filename, MAX_PATH, "-", NULL);
-      }
+      OemToCharBuff(filename, filename, strlen(filename));
     }
 
     if (filename[0] &&
@@ -237,6 +243,19 @@ int tempzip_make(HWND hwndDlg, char *fn)
           } while (l > 0);
 
           fclose(fp);
+
+          {
+            // set file time
+            HANDLE hf = CreateFile(out_filename, GENERIC_WRITE, 0, 0, OPEN_ALWAYS, 0, 0);
+            if (hf != INVALID_HANDLE_VALUE)
+            {
+              FILETIME ft, lft;
+              DosDateTimeToFileTime(HIWORD(info.dosDate), LOWORD(info.dosDate), &ft);
+              LocalFileTimeToFileTime(&ft, &lft);
+              SetFileTime(hf, 0, 0, &lft);
+              CloseHandle(hf);
+            }
+          }
         }
         else
         {
@@ -467,6 +486,8 @@ void makeEXE(HWND hwndDlg)
     fprintf(fp,"!define ZIP2EXE_COMPRESSOR_BZIP2\n");
   if (g_compressor == 3)
     fprintf(fp,"!define ZIP2EXE_COMPRESSOR_LZMA\n");
+  if (g_compressor_solid == 1)
+    fprintf(fp,"!define ZIP2EXE_COMPRESSOR_SOLID\n");
   GetDlgItemText(hwndDlg,IDC_INSTPATH,buf,sizeof(buf));
   char *outpath = "$INSTDIR";
   int iswinamp=0;
@@ -557,11 +578,34 @@ void makeEXE(HWND hwndDlg)
 
 }
 
+void SetZip(HWND hwndDlg, char *path)
+{
+  char buf2[1024];
+  lstrcpy(buf2,path);
+  tempzip_cleanup(hwndDlg,1);
+  SetDlgItemText(hwndDlg,IDC_ZIPFILE,path);
+  char *t=path+lstrlen(path);
+  while (t > path && *t != '\\' && *t != '.') t--;
+  {
+    char *p=t;
+    while (p >= path && *p != '\\') p--;
+    p++;
+    *t=0;
+    SetDlgItemText(hwndDlg,IDC_INSTNAME,p[0]?p:"Stuff");
+  }
+  strcpy(t,".exe");
+  SetDlgItemText(hwndDlg,IDC_OUTFILE,path);
+  if (tempzip_make(hwndDlg,buf2)) tempzip_cleanup(hwndDlg,1);
+  else
+  {
+    EnableWindow(GetDlgItem(hwndDlg,IDOK),1);
+  }
+}
 
 BOOL CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   static int ids[]={IDC_INFO,IDC_NSISICON,IDC_SZIPFRAME,IDC_BROWSE,IDC_ZIPFILE,IDC_ZIPINFO_SUMMARY,IDC_ZIPINFO_FILES,IDC_OFRAME,IDC_INAMEST,
-                        IDC_INSTNAME,IDC_INSTPATH,IDC_OEFST,IDC_OUTFILE,IDC_BROWSE2,IDC_COMPRESSOR,IDC_ZLIB,IDC_BZIP2,IDC_LZMA,IDC_INTERFACE,IDC_MODERNUI,IDC_CLASSICUI};
+                        IDC_INSTNAME,IDC_INSTPATH,IDC_OEFST,IDC_OUTFILE,IDC_BROWSE2,IDC_COMPRESSOR,IDC_ZLIB,IDC_BZIP2,IDC_LZMA,IDC_SOLID,IDC_INTERFACE,IDC_MODERNUI,IDC_CLASSICUI};
   static HICON hIcon;
   static HFONT hFont;
   if (uMsg == WM_DESTROY) { if (hIcon) DeleteObject(hIcon); hIcon=0; if (hFont) DeleteObject(hFont); hFont=0; }
@@ -597,6 +641,8 @@ BOOL CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
               CLIP_DEFAULT_PRECIS,
               DEFAULT_QUALITY,FIXED_PITCH|FF_DONTCARE,"Courier New");
       SendDlgItemMessage(hwndDlg,IDC_OUTPUTTEXT,WM_SETFONT,(WPARAM)hFont,0);
+
+      DragAcceptFiles(hwndDlg,TRUE);
     return 1;
     case WM_CLOSE:
       if (!g_hThread)
@@ -619,6 +665,24 @@ BOOL CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       if (nsifilename[0]) DeleteFile(nsifilename);
       nsifilename[0]=0;
     break;
+    case WM_DROPFILES:
+    {
+      char dropped_file[MAX_PATH]="";
+      if (DragQueryFile((HDROP)wParam,-1,NULL,0)==1)
+      {
+        DragQueryFile((HDROP)wParam,0,dropped_file,MAX_PATH);
+        if (lstrlen(dropped_file)>0)
+        {
+          SetZip(hwndDlg,dropped_file);
+        }
+      }
+      else
+      {
+        MessageBox(hwndDlg,"Dropping more than one zip file at a time is not supported",g_errcaption,MB_OK|MB_ICONSTOP);
+      }
+      DragFinish((HDROP)wParam);
+      return TRUE;
+    }
     case WM_COMMAND:
       switch (LOWORD(wParam))
       {
@@ -637,26 +701,7 @@ BOOL CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
             buf[0]=0;
             if (GetOpenFileName(&l))
             {
-              char buf2[1024];
-              lstrcpy(buf2,buf);
-              tempzip_cleanup(hwndDlg,1);
-              SetDlgItemText(hwndDlg,IDC_ZIPFILE,buf);
-              char *t=buf+strlen(buf);
-              while (t > buf && *t != '\\' && *t != '.') t--;
-              {
-                char *p=t;
-                while (p >= buf && *p != '\\') p--;
-                p++;
-                *t=0;
-                SetDlgItemText(hwndDlg,IDC_INSTNAME,p[0]?p:"Stuff");
-              }
-              strcpy(t,".exe");
-              SetDlgItemText(hwndDlg,IDC_OUTFILE,buf);
-              if (tempzip_make(hwndDlg,buf2)) tempzip_cleanup(hwndDlg,1);
-              else
-              {
-                EnableWindow(GetDlgItem(hwndDlg,IDOK),1);
-              }
+              SetZip(hwndDlg,buf);
             }
           }
         break;
@@ -713,6 +758,10 @@ BOOL CALLBACK DlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
                 g_compressor = 2;
               if (IsDlgButtonChecked(hwndDlg,IDC_LZMA))
                 g_compressor = 3;
+              if (IsDlgButtonChecked(hwndDlg,IDC_SOLID))
+                g_compressor_solid = 1;
+              else
+                g_compressor_solid = 0;
               g_mui=!IsDlgButtonChecked(hwndDlg,IDC_CLASSICUI);
               SetDlgItemText(g_hwnd, IDC_OUTPUTTEXT, "");
               int x;
