@@ -21,30 +21,45 @@
 */
 
 #include "DialogTemplate.h"
-#include "util.h"
+#include <cassert> // for assert(3)
 #ifndef _WIN32
+#  include "util.h" // for Unicode conversion functions
 #  include <stdio.h>
 #  include <stdlib.h>
 #  include <iconv.h>
 #  include <errno.h>
 #endif
 
+using namespace std;
+
 //////////////////////////////////////////////////////////////////////
 // Utilities
 //////////////////////////////////////////////////////////////////////
 
+static inline DWORD ConvertEndianness(DWORD d) {
+  return FIX_ENDIAN_INT32(d);
+}
+
+static inline WORD ConvertEndianness(WORD w) {
+  return FIX_ENDIAN_INT16(w);
+}
+
+static inline short ConvertEndianness(short s) {
+  return ConvertEndianness(WORD(s));
+}
+
 #define ALIGN(dwToAlign, dwAlignOn) dwToAlign = (dwToAlign%dwAlignOn == 0) ? dwToAlign : dwToAlign - (dwToAlign%dwAlignOn) + dwAlignOn
 
 // Reads a variany length array from seeker into readInto and advances seeker
-void ReadVarLenArr(BYTE* &seeker, char* &readInto, unsigned int uCodePage) {
+void ReadVarLenArr(LPBYTE &seeker, char* &readInto, unsigned int uCodePage) {
   WORD* arr = (WORD*)seeker;
-  switch (arr[0]) {
+  switch (ConvertEndianness(arr[0])) {
   case 0x0000:
     readInto = 0;
     seeker += sizeof(WORD);
     break;
   case 0xFFFF:
-    readInto = MAKEINTRESOURCE(arr[1]);
+    readInto = MAKEINTRESOURCE(ConvertEndianness(arr[1]));
     seeker += 2*sizeof(WORD);
     break;
   default:
@@ -63,7 +78,10 @@ void ReadVarLenArr(BYTE* &seeker, char* &readInto, unsigned int uCodePage) {
       {
         throw runtime_error("ReadVarLenArr - Unicode conversion failed.");
       }
-      seeker += iStrLen * sizeof(WCHAR);
+
+      PWCHAR wseeker = PWCHAR(seeker);
+      while (*wseeker++);
+      seeker = LPBYTE(wseeker);
     }
     break;
   }
@@ -75,11 +93,11 @@ void ReadVarLenArr(BYTE* &seeker, char* &readInto, unsigned int uCodePage) {
     if (IS_INTRESOURCE(x)) { \
       *(WORD*)seeker = 0xFFFF; \
       seeker += sizeof(WORD); \
-      *(WORD*)seeker = WORD(DWORD(x)); \
+      *(WORD*)seeker = ConvertEndianness(WORD(DWORD(x))); \
       seeker += sizeof(WORD); \
     } \
     else { \
-      int us = MultiByteToWideChar(m_uCodePage, 0, x, -1, (WCHAR*)seeker, dwSize); \
+      int us = MultiByteToWideChar(m_uCodePage, 0, x, -1, (WCHAR*)seeker, dwSize - DWORD(seeker - pbDlg)); \
       if (!us) { \
         throw runtime_error("WriteStringOrId - Unicode conversion failed."); \
       } \
@@ -89,7 +107,7 @@ void ReadVarLenArr(BYTE* &seeker, char* &readInto, unsigned int uCodePage) {
     seeker += sizeof(WORD);
 
 // A macro that adds the size of x (which can be a string a number, or nothing) to dwSize
-#define AddStringOrIdSize(x) dwSize += x ? (IS_INTRESOURCE(x) ? sizeof(DWORD) : (strlen(x)+1)*sizeof(WCHAR)) : sizeof(WORD)
+#define AddStringOrIdSize(x) dwSize += x ? (IS_INTRESOURCE(x) ? sizeof(DWORD) : MultiByteToWideChar(m_uCodePage, 0, x, -1, 0, 0) * sizeof(WCHAR)) : sizeof(WORD)
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -98,8 +116,13 @@ void ReadVarLenArr(BYTE* &seeker, char* &readInto, unsigned int uCodePage) {
 CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
   m_uCodePage = uCodePage;
 
+  m_dwHelpId = 0;
   m_szClass = 0;
   m_szFont = 0;
+  m_sFontSize = 0;
+  m_sFontWeight = 0;
+  m_bItalic = 0;
+  m_bCharset = 0;
   m_szMenu = 0;
   m_szTitle = 0;
 
@@ -110,29 +133,29 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
 
     DLGTEMPLATEEX* dTemplateEx = (DLGTEMPLATEEX*)pbData;
 
-    m_dwHelpId = dTemplateEx->helpID;
-    m_dwStyle = dTemplateEx->style;
-    m_dwExtStyle = dTemplateEx->exStyle;
-    m_sX = dTemplateEx->x;
-    m_sY = dTemplateEx->y;
-    m_sWidth = dTemplateEx->cx;
-    m_sHeight = dTemplateEx->cy;
+    m_dwHelpId = ConvertEndianness(dTemplateEx->helpID);
+    m_dwStyle = ConvertEndianness(dTemplateEx->style);
+    m_dwExtStyle = ConvertEndianness(dTemplateEx->exStyle);
+    m_sX = ConvertEndianness(dTemplateEx->x);
+    m_sY = ConvertEndianness(dTemplateEx->y);
+    m_sWidth = ConvertEndianness(dTemplateEx->cx);
+    m_sHeight = ConvertEndianness(dTemplateEx->cy);
 
-    wItems = dTemplateEx->cDlgItems;
+    wItems = ConvertEndianness(dTemplateEx->cDlgItems);
   }
   else {
     m_bExtended = false;
 
     DLGTEMPLATE* dTemplate = (DLGTEMPLATE*)pbData;
 
-    m_dwStyle = dTemplate->style;
-    m_dwExtStyle = dTemplate->dwExtendedStyle;
-    m_sX = dTemplate->x;
-    m_sY = dTemplate->y;
-    m_sWidth = dTemplate->cx;
-    m_sHeight = dTemplate->cy;
+    m_dwStyle = ConvertEndianness(dTemplate->style);
+    m_dwExtStyle = ConvertEndianness(dTemplate->dwExtendedStyle);
+    m_sX = ConvertEndianness(dTemplate->x);
+    m_sY = ConvertEndianness(dTemplate->y);
+    m_sWidth = ConvertEndianness(dTemplate->cx);
+    m_sHeight = ConvertEndianness(dTemplate->cy);
 
-    wItems = dTemplate->cdit;
+    wItems = ConvertEndianness(dTemplate->cdit);
   }
 
   BYTE* seeker = pbData + (m_bExtended ? sizeof(DLGTEMPLATEEX) : sizeof(DLGTEMPLATE));
@@ -145,10 +168,10 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
   ReadVarLenArr(seeker, m_szTitle, m_uCodePage);
   // Read font size and variant length array (only if style DS_SETFONT is used!)
   if (m_dwStyle & DS_SETFONT) {
-    m_sFontSize = *(short*)seeker;
+    m_sFontSize = ConvertEndianness(*(short*)seeker);
     seeker += sizeof(short);
     if (m_bExtended) {
-      m_sFontWeight = *(short*)seeker;
+      m_sFontWeight = ConvertEndianness(*(short*)seeker);
       seeker += sizeof(short);
       m_bItalic = *(BYTE*)seeker;
       seeker += sizeof(BYTE);
@@ -170,27 +193,28 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
     if (m_bExtended) {
       DLGITEMTEMPLATEEX* rawItem = (DLGITEMTEMPLATEEX*)seeker;
 
-      item->dwHelpId = rawItem->helpID;
-      item->dwStyle = rawItem->style;
-      item->dwExtStyle = rawItem->exStyle;
-      item->sX = rawItem->x;
-      item->sY = rawItem->y;
-      item->sWidth = rawItem->cx;
-      item->sHeight = rawItem->cy;
-      item->wId = rawItem->id;
+      item->dwHelpId = ConvertEndianness(rawItem->helpID);
+      item->dwStyle = ConvertEndianness(rawItem->style);
+      item->dwExtStyle = ConvertEndianness(rawItem->exStyle);
+      item->sX = ConvertEndianness(rawItem->x);
+      item->sY = ConvertEndianness(rawItem->y);
+      item->sWidth = ConvertEndianness(rawItem->cx);
+      item->sHeight = ConvertEndianness(rawItem->cy);
+      item->wId = ConvertEndianness(rawItem->id);
 
       seeker += sizeof(DLGITEMTEMPLATEEX);
     }
     else {
       DLGITEMTEMPLATE* rawItem = (DLGITEMTEMPLATE*)seeker;
 
-      item->dwStyle = rawItem->style;
-      item->dwExtStyle = rawItem->dwExtendedStyle;
-      item->sX = rawItem->x;
-      item->sY = rawItem->y;
-      item->sWidth = rawItem->cx;
-      item->sHeight = rawItem->cy;
-      item->wId = rawItem->id;
+      item->dwHelpId = 0;
+      item->dwStyle = ConvertEndianness(rawItem->style);
+      item->dwExtStyle = ConvertEndianness(rawItem->dwExtendedStyle);
+      item->sX = ConvertEndianness(rawItem->x);
+      item->sY = ConvertEndianness(rawItem->y);
+      item->sWidth = ConvertEndianness(rawItem->cx);
+      item->sHeight = ConvertEndianness(rawItem->cy);
+      item->wId = ConvertEndianness(rawItem->id);
 
       seeker += sizeof(DLGITEMTEMPLATE);
     }
@@ -202,7 +226,7 @@ CDialogTemplate::CDialogTemplate(BYTE* pbData, unsigned int uCodePage) {
 
     // Read creation data variant length array
     // First read the size of the array (no null termination)
-    item->wCreateDataSize = *(WORD*)seeker;
+    item->wCreateDataSize = ConvertEndianness(*(WORD*)seeker);
     seeker += sizeof(WORD);
     // Then read the array it self (if size is not 0)
     if (item->wCreateDataSize) {
@@ -287,6 +311,7 @@ void CDialogTemplate::SetFont(char* szFaceName, WORD wFontSize) {
     // MS Shell Dlg
     m_dwStyle |= DS_SHELLFONT;
   }
+  m_bCharset = DEFAULT_CHARSET;
   m_dwStyle |= DS_SETFONT;
   if (m_szFont) delete [] m_szFont;
   m_szFont = new char[strlen(szFaceName)+1];
@@ -437,7 +462,7 @@ void CDialogTemplate::ConvertToRTL() {
     bool addExStyle = false;
 
     // Button
-    if (int(m_vItems[i]->szClass) == 0x80) {
+    if (long(m_vItems[i]->szClass) == 0x80) {
       m_vItems[i]->dwStyle ^= BS_LEFTTEXT;
       m_vItems[i]->dwStyle ^= BS_RIGHT;
       m_vItems[i]->dwStyle ^= BS_LEFT;
@@ -451,13 +476,13 @@ void CDialogTemplate::ConvertToRTL() {
       }
     }
     // Edit
-    else if (int(m_vItems[i]->szClass) == 0x81) {
+    else if (long(m_vItems[i]->szClass) == 0x81) {
       if ((m_vItems[i]->dwStyle & ES_CENTER) == 0) {
         m_vItems[i]->dwStyle ^= ES_RIGHT;
       }
     }
     // Static
-    else if (int(m_vItems[i]->szClass) == 0x82) {
+    else if (long(m_vItems[i]->szClass) == 0x82) {
       if ((m_vItems[i]->dwStyle & SS_TYPEMASK) == SS_LEFT || (m_vItems[i]->dwStyle & SS_TYPEMASK) == SS_LEFTNOWORDWRAP)
       {
         m_vItems[i]->dwStyle &= ~SS_TYPEMASK;
@@ -481,11 +506,11 @@ void CDialogTemplate::ConvertToRTL() {
     if (addExStyle)
       m_vItems[i]->dwExtStyle |= WS_EX_RIGHT;
 
-    m_vItems[i]->dwExtStyle |= WS_EX_RTLREADING;
+    m_vItems[i]->dwExtStyle |= WS_EX_RTLREADING | WS_EX_LEFTSCROLLBAR;
 
     m_vItems[i]->sX = m_sWidth - m_vItems[i]->sWidth - m_vItems[i]->sX;
   }
-  m_dwExtStyle |= WS_EX_RIGHT | WS_EX_RTLREADING;
+  m_dwExtStyle |= WS_EX_RIGHT | WS_EX_RTLREADING | WS_EX_LEFTSCROLLBAR;
 }
 
 // Saves the dialog in the form of DLGTEMPLATE[EX]
@@ -498,16 +523,16 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
 
   if (m_bExtended) {
     DLGTEMPLATEEX dh = {
-      0x0001,
-      0xFFFF,
-      m_dwHelpId,
-      m_dwExtStyle,
-      m_dwStyle,
-      m_vItems.size(),
-      m_sX,
-      m_sY,
-      m_sWidth,
-      m_sHeight
+      ConvertEndianness(WORD(0x0001)),
+      ConvertEndianness(WORD(0xFFFF)),
+      ConvertEndianness(m_dwHelpId),
+      ConvertEndianness(m_dwExtStyle),
+      ConvertEndianness(m_dwStyle),
+      ConvertEndianness(WORD(m_vItems.size())),
+      ConvertEndianness(m_sX),
+      ConvertEndianness(m_sY),
+      ConvertEndianness(m_sWidth),
+      ConvertEndianness(m_sHeight)
     };
 
     CopyMemory(seeker, &dh, sizeof(DLGTEMPLATEEX));
@@ -515,13 +540,13 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
   }
   else {
     DLGTEMPLATE dh = {
-      m_dwStyle,
-      m_dwExtStyle,
-      m_vItems.size(),
-      m_sX,
-      m_sY,
-      m_sWidth,
-      m_sHeight
+      ConvertEndianness(m_dwStyle),
+      ConvertEndianness(m_dwExtStyle),
+      ConvertEndianness(WORD(m_vItems.size())),
+      ConvertEndianness(m_sX),
+      ConvertEndianness(m_sY),
+      ConvertEndianness(m_sWidth),
+      ConvertEndianness(m_sHeight)
     };
 
     CopyMemory(seeker, &dh, sizeof(DLGTEMPLATE));
@@ -537,10 +562,10 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
 
   // Write font variant length array, size, and extended info (if needed)
   if (m_dwStyle & DS_SETFONT) {
-    *(short*)seeker = m_sFontSize;
+    *(short*)seeker = ConvertEndianness(m_sFontSize);
     seeker += sizeof(short);
     if (m_bExtended) {
-      *(short*)seeker = m_sFontWeight;
+      *(short*)seeker = ConvertEndianness(m_sFontWeight);
       seeker += sizeof(short);
       *(BYTE*)seeker = m_bItalic;
       seeker += sizeof(BYTE);
@@ -558,14 +583,14 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
 
     if (m_bExtended) {
       DLGITEMTEMPLATEEX dih = {
-        m_vItems[i]->dwHelpId,
-        m_vItems[i]->dwExtStyle,
-        m_vItems[i]->dwStyle,
-        m_vItems[i]->sX,
-        m_vItems[i]->sY,
-        m_vItems[i]->sWidth,
-        m_vItems[i]->sHeight,
-        m_vItems[i]->wId
+        ConvertEndianness(m_vItems[i]->dwHelpId),
+        ConvertEndianness(m_vItems[i]->dwExtStyle),
+        ConvertEndianness(m_vItems[i]->dwStyle),
+        ConvertEndianness(m_vItems[i]->sX),
+        ConvertEndianness(m_vItems[i]->sY),
+        ConvertEndianness(m_vItems[i]->sWidth),
+        ConvertEndianness(m_vItems[i]->sHeight),
+        ConvertEndianness(m_vItems[i]->wId)
       };
 
       CopyMemory(seeker, &dih, sizeof(DLGITEMTEMPLATEEX));
@@ -573,13 +598,13 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     }
     else {
       DLGITEMTEMPLATE dih = {
-        m_vItems[i]->dwStyle,
-        m_vItems[i]->dwExtStyle,
-        m_vItems[i]->sX,
-        m_vItems[i]->sY,
-        m_vItems[i]->sWidth,
-        m_vItems[i]->sHeight,
-        m_vItems[i]->wId
+        ConvertEndianness(m_vItems[i]->dwStyle),
+        ConvertEndianness(m_vItems[i]->dwExtStyle),
+        ConvertEndianness(m_vItems[i]->sX),
+        ConvertEndianness(m_vItems[i]->sY),
+        ConvertEndianness(m_vItems[i]->sWidth),
+        ConvertEndianness(m_vItems[i]->sHeight),
+        ConvertEndianness(m_vItems[i]->wId)
       };
 
       CopyMemory(seeker, &dih, sizeof(DLGITEMTEMPLATE));
@@ -595,7 +620,7 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     // First write its size
     WORD wCreateDataSize = m_vItems[i]->wCreateDataSize;
     if (m_vItems[i]->wCreateDataSize) wCreateDataSize += sizeof(WORD);
-    *(WORD*)seeker = wCreateDataSize;
+    *(WORD*)seeker = ConvertEndianness(wCreateDataSize);
     seeker += sizeof(WORD);
     // If size is nonzero write the data too
     if (m_vItems[i]->wCreateDataSize) {
@@ -604,6 +629,8 @@ BYTE* CDialogTemplate::Save(DWORD& dwSize) {
     }
   }
 
+  assert((DWORD) seeker - (DWORD) pbDlg == dwSize);
+
   // DONE!
   return pbDlg;
 }
