@@ -5,6 +5,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+using namespace std; // for std::min
+
 int g_display_errors = 1;
 FILE *g_output = stderr;
 
@@ -39,7 +41,8 @@ public:
 
       int offset2 = rand() % BUF_SIZE;
       int size2 = rand() % (BUF_SIZE - offset2);
-      char *p2 = (char *) mmap.getmore(offset2, &size2);
+      int size2forrelease = size2;
+      char *p2 = (char *) mmap.getmore(offset2, &size2forrelease);
 
       int minsize = min(size1, size2);
       for (size_t j = 0; j < minsize; j++) {
@@ -47,7 +50,7 @@ public:
       }
 
       mmap.release();
-      mmap.release(p2, size2);
+      mmap.release(p2, size2forrelease);
     }
   }
 
