@@ -30,9 +30,11 @@ exec_flags g_exec_flags;
 struct {
   exec_flags *flags;
   void *ExecuteCodeSegment;
+  void *validate_filename;
 } plugin_extra_parameters = {
   &g_exec_flags,
-  &ExecuteCodeSegment
+  &ExecuteCodeSegment,
+  &validate_filename
 };
 
 #if defined(NSIS_SUPPORT_ACTIVEXREG) || defined(NSIS_SUPPORT_CREATESHORTCUT)
@@ -287,16 +289,20 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         {
           while (c)
           {
-            WIN32_FIND_DATA *fd;
             p = findchar(p, '\\');
-            c=*p;
-            *p=0;
-            fd = file_exists(buf1);
-            if (!fd) {
-              if (!CreateDirectory(buf1,NULL))
+            c = *p;
+            *p = 0;
+            if (!CreateDirectory(buf1, NULL))
+            {
+              if (GetLastError() != ERROR_ALREADY_EXISTS)
+              {
                 exec_error++;
+              }
+              else if ((GetFileAttributes(buf1) & FILE_ATTRIBUTE_DIRECTORY) == 0)
+              {
+                exec_error++;
+              }
             }
-            else if ((fd->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0) exec_error++;
             *p++ = c;
           }
         }
@@ -329,8 +335,8 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         mystrcpy(buf1,buf3);
         if (mystrlen(buf3)+mystrlen(buf2) < NSIS_MAX_STRLEN-3)
         {
-          lstrcat(buf1,"->");
-          lstrcat(buf1,buf2);
+          mystrcat(buf1,"->");
+          mystrcat(buf1,buf2);
         }
         log_printf2("Rename: %s",buf1);
         if (MoveFile(buf3,buf2))
@@ -416,7 +422,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         {
           mystrcpy(buf0,buf3);
         }
-        else lstrcat(addtrailingslash(mystrcpy(buf0,state_output_directory)),buf3);
+        else mystrcat(addtrailingslash(mystrcpy(buf0,state_output_directory)),buf3);
         validate_filename(buf0);
       _tryagain:
         if (overwriteflag >= 3) // check date and time
@@ -442,7 +448,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
           if (overwriteflag)
           {
             update_status_text(LANG_SKIPPED,buf3);
-            if (overwriteflag==2) g_exec_flags.exec_error++;
+            if (overwriteflag==2) exec_error++;
             log_printf3("File: skipped: \"%s\" (overwriteflag=%d)",buf0,overwriteflag);
             break;
           }
@@ -489,7 +495,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
           if (ret == -2)
           {
             GetNSISString(buf0,LANG_ERRORWRITING);
-            lstrcat(buf0,buf3);
+            mystrcat(buf0,buf3);
           }
           else
           {
@@ -577,10 +583,18 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         }
       }
     break;
-    case EW_STRCMP: {
+    case EW_STRCMP:
+    {
       char *buf2=GetStringFromParm(0x20);
       char *buf3=GetStringFromParm(0x31);
-      if (!lstrcmpi(buf2,buf3)) return parm2;
+      if (!parm4) {
+        // case insensitive
+        if (!lstrcmpi(buf2,buf3)) return parm2;
+      }
+      else {
+        // case sensitive
+        if (!lstrcmp(buf2,buf3)) return parm2;
+      }
     }
     return parm3;
 #endif//NSIS_SUPPORT_STROPTS
@@ -599,7 +613,11 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         }
         else
         {
-          ExpandEnvironmentStrings(buf0,p,NSIS_MAX_STRLEN);
+          if (!ExpandEnvironmentStrings(buf0,p,NSIS_MAX_STRLEN))
+          {
+            exec_error++;
+            *p=0;
+          }
         }
         p[NSIS_MAX_STRLEN-1]=0;
       }
@@ -688,7 +706,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         }
         else
         {
-          s=(stack_t*)my_GlobalAlloc(sizeof(stack_t));
+          s=(stack_t*)GlobalAlloc(GPTR,sizeof(stack_t));
           GetNSISString(s->text,parm0);
           s->next=g_st;
           g_st=s;
@@ -834,9 +852,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
             DWORD lExitCode;
             while (WaitForSingleObject(hProc,100) == WAIT_TIMEOUT)
             {
-              MSG msg;
-              while (PeekMessage(&msg,NULL,WM_PAINT,WM_PAINT,PM_REMOVE))
-                DispatchMessage(&msg);
+              MessageLoop(WM_PAINT);
             }
             GetExitCodeProcess(hProc, &lExitCode);
 
@@ -894,7 +910,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         if (s1)
         {
           void *b1;
-          b1=my_GlobalAlloc(s1);
+          b1=GlobalAlloc(GPTR,s1);
           if (b1)
           {
             UINT uLen;
@@ -1049,7 +1065,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         buf1[mystrlen(buf1)+1]=0;
 
         GetNSISString(buf2,LANG_COPYTO);
-        lstrcat(buf2,buf1);
+        mystrcat(buf2,buf1);
 
         op.pFrom=buf0;
         op.pTo=buf1;
@@ -1123,6 +1139,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
     case EW_DELREG:
       {
         long res=!ERROR_SUCCESS;
+        const char *rkn=RegKeyHandleToName((HKEY)parm1);
         if (!parm4)
         {
           HKEY hKey=myRegOpenKey(KEY_SET_VALUE);
@@ -1130,14 +1147,14 @@ static int NSISCALL ExecuteEntry(entry *entry_)
           {
             char *buf3=GetStringFromParm(0x33);
             res = RegDeleteValue(hKey,buf3);
-            log_printf4("DeleteRegValue: %d\\%s\\%s",parm1,buf2,buf3);
+            log_printf4("DeleteRegValue: \"%s\\%s\" \"%s\"",rkn,buf2,buf3);
             RegCloseKey(hKey);
           }
         }
         else
         {
           char *buf2=GetStringFromParm(0x22);
-          log_printf3("DeleteRegKey: %d\\%s",parm1,buf2);
+          log_printf3("DeleteRegKey: \"%s\\%s\"",rkn,buf2);
           res = myRegDeleteKeyEx(GetRegRootKey(parm1),buf2,parm4&2);
         }
         if (res != ERROR_SUCCESS)
@@ -1152,6 +1169,8 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         int rtype=parm5;
         char *buf0=GetStringFromParm(0x02);
         char *buf1=GetStringFromParm(0x11);
+        const char *rkn=RegKeyHandleToName(rootkey);
+
         exec_error++;
         if (RegCreateKeyEx(rootkey,buf1,0,0,REG_OPTION_NON_VOLATILE,KEY_SET_VALUE,0,&hKey,0) == ERROR_SUCCESS)
         {
@@ -1161,25 +1180,36 @@ static int NSISCALL ExecuteEntry(entry *entry_)
           {
             GetStringFromParm(0x23);
             size = mystrlen((char *) data) + 1;
-            log_printf5("WriteRegStr: set %d\\%s\\%s to %s",rootkey,buf1,buf0,data);
+            if (rtype == REG_SZ)
+            {
+              log_printf5("WriteRegStr: \"%s\\%s\" \"%s\"=\"%s\"",rkn,buf1,buf0,data);
+            }
+            else
+            {
+              log_printf5("WriteRegExpandStr: \"%s\\%s\" \"%s\"=\"%s\"",rkn,buf1,buf0,data);
+            }
           }
           if (type == REG_DWORD)
           {
             *(LPDWORD) data = GetIntFromParm(3);
             size = sizeof(DWORD);
-            log_printf5("WriteRegDWORD: set %d\\%s\\%s to %d",rootkey,buf1,buf0,*(LPDWORD)data);
+            log_printf5("WriteRegDWORD: \"%s\\%s\" \"%s\"=\"0x%08x\"",rkn,buf1,buf0,*(LPDWORD) data);
           }
           if (type == REG_BINARY)
           {
+#ifdef NSIS_CONFIG_LOG
+            char binbuf[128];
+#endif
             // use buf2, buf3 and buf4
             size = GetCompressedDataFromDataBlockToMemory(parm3, data, 3 * NSIS_MAX_STRLEN);
-            log_printf5("WriteRegBin: set %d\\%s\\%s with %d bytes",rootkey,buf1,buf0,size);
+            LogData2Hex(binbuf, sizeof(binbuf), data, size);
+            log_printf5("WriteRegBin: \"%s\\%s\" \"%s\"=\"%s\"",rkn,buf1,buf0,binbuf);
           }
           if (size >= 0 && RegSetValueEx(hKey,buf0,0,rtype,data,size) == ERROR_SUCCESS)
             exec_error--;
           RegCloseKey(hKey);
         }
-        else { log_printf3("WriteReg: error creating key %d\\%s",rootkey,buf1); }
+        else { log_printf3("WriteReg: error creating key \"%s\\%s\"",buf3,buf1); }
       }
     break;
     case EW_READREGSTR: // read registry string
@@ -1190,7 +1220,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         p[0]=0;
         if (hKey)
         {
-          DWORD l = NSIS_MAX_STRLEN;
+          DWORD l = NSIS_MAX_STRLEN - 1;
           DWORD t;
 
           if (RegQueryValueEx(hKey,buf3,NULL,&t,p,&l) != ERROR_SUCCESS ||
@@ -1206,7 +1236,11 @@ static int NSISCALL ExecuteEntry(entry *entry_)
               if (!parm4) exec_error++;
               myitoa(p,*((DWORD*)p));
             }
-            else if (parm4) exec_error++;
+            else
+            {
+              if (parm4) exec_error++;
+              p[l]=0;
+            }
           }
           RegCloseKey(hKey);
         }
@@ -1223,7 +1257,11 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         {
           DWORD d=NSIS_MAX_STRLEN-1;
           if (parm4) RegEnumKey(key,b,p,d);
-          else RegEnumValue(key,b,p,&d,NULL,NULL,NULL,NULL);
+          else if (RegEnumValue(key,b,p,&d,NULL,NULL,NULL,NULL)!=ERROR_SUCCESS)
+          {
+            exec_error++;
+            break;
+          }
           p[NSIS_MAX_STRLEN-1]=0;
           RegCloseKey(key);
         }
@@ -1387,7 +1425,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         }
         else
         {
-          lstrcat(addtrailingslash(mystrcpy(buf1,state_install_directory)),buf0);
+          mystrcat(addtrailingslash(mystrcpy(buf1,state_install_directory)),buf0);
         }
         validate_filename(buf1);
 
@@ -1396,7 +1434,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         {
           unsigned char *filebuf;
           int filehdrsize = g_filehdrsize;
-          filebuf=(unsigned char *)my_GlobalAlloc(filehdrsize);
+          filebuf=(unsigned char *)GlobalAlloc(GPTR,filehdrsize);
           if (filebuf)
           {
             DWORD lout;
@@ -1404,7 +1442,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
             ReadSelfFile((char*)filebuf,filehdrsize);
             {
               unsigned char* seeker;
-              unsigned char* unicon_data = seeker = (unsigned char*)my_GlobalAlloc(parm2);
+              unsigned char* unicon_data = seeker = (unsigned char*)GlobalAlloc(GPTR,parm2);
               if (unicon_data) {
                 GetCompressedDataFromDataBlockToMemory(parm1,unicon_data,parm2);
                 while (*seeker) {
@@ -1446,7 +1484,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         log_printf2("settings logging to %d",parm1);
         log_dolog=parm1;
         log_printf2("logging set to %d",parm1);
-#ifndef NSIS_CONFIG_LOG_ODS
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
         if (parm1) build_g_logfile();
 #endif
       }
