@@ -9,7 +9,7 @@
 #include "ui.h"
 
 #ifdef NSIS_CONFIG_LOG
-#ifndef NSIS_CONFIG_LOG_ODS
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
 char g_log_file[1024];
 #endif
 #endif
@@ -21,13 +21,13 @@ char g_log_file[1024];
 // which result in extra memory for extra variables without code to do allocation :)
 // nsis then removes the "DISCARDABLE" style from section (for safe)
 #ifdef _MSC_VER
-#  pragma bss_seg(VARS_SECTION_NAME)
+#  pragma bss_seg(NSIS_VARS_SECTION)
 NSIS_STRING g_usrvars[1];
 #  pragma bss_seg()
-#  pragma comment(linker, "/section:" VARS_SECTION_NAME ",rwd")
+#  pragma comment(linker, "/section:" NSIS_VARS_SECTION ",rwd")
 #else
 #  ifdef __GNUC__
-NSIS_STRING g_usrvars[1] __attribute__((section (VARS_SECTION_NAME)));
+NSIS_STRING g_usrvars[1] __attribute__((section (NSIS_VARS_SECTION)));
 #  else
 #    error Unknown compiler. You must implement the seperate PE section yourself.
 #  endif
@@ -90,10 +90,6 @@ int NSISCALL my_MessageBox(const char *text, UINT type) {
   return MessageBox(g_hwnd, text, g_caption, _type);
 }
 
-void * NSISCALL my_GlobalAlloc(DWORD dwBytes) {
-  return (void *)GlobalAlloc(GPTR, dwBytes);
-}
-
 void NSISCALL myDelete(char *buf, int flags)
 {
   static char lbuf[NSIS_MAX_STRLEN];
@@ -116,12 +112,12 @@ void NSISCALL myDelete(char *buf, int flags)
     mystrcpy(lbuf,buf);
 #ifdef NSIS_SUPPORT_RMDIR
     if (flags & DEL_DIR)
-      lstrcat(lbuf,"\\*.*");
+      mystrcat(lbuf,"\\*.*");
     else
 #endif//NSIS_SUPPORT_RMDIR
       trimslashtoend(buf);
 
-    lstrcat(buf,"\\");
+    mystrcat(buf,"\\");
 
     fn=buf+mystrlen(buf);
 
@@ -130,12 +126,17 @@ void NSISCALL myDelete(char *buf, int flags)
     {
       do
       {
+        char *fdfn = fd.cFileName;
+        if (*findchar(fdfn, '?') && *fd.cAlternateFileName)
+          // name contains unicode, use short name
+          fdfn = fd.cAlternateFileName;
+
 #ifdef NSIS_SUPPORT_RMDIR
-        if (fd.cFileName[0] != '.' ||
-            (fd.cFileName[1] != '.' && fd.cFileName[1]))
+        if (fdfn[0] == '.' && !fdfn[1]) continue;
+        if (fdfn[0] == '.' && fdfn[1] == '.' && !fdfn[2]) continue;
 #endif//NSIS_SUPPORT_RMDIR
         {
-          mystrcpy(fn,fd.cFileName);
+          mystrcpy(fn,fdfn);
           if (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
           {
 #ifdef NSIS_SUPPORT_RMDIR
@@ -180,35 +181,46 @@ void NSISCALL myDelete(char *buf, int flags)
   }
 
 #ifdef NSIS_SUPPORT_RMDIR
-  if (valid_dir && (flags & DEL_DIR))
+  if ((flags & DEL_DIR))
   {
-    addtrailingslash(buf);
-    log_printf2("RMDir: RemoveDirectory(\"%s\")",buf);
-    if (!RemoveDirectory(buf))
+    if (!valid_dir)
     {
-#ifdef NSIS_SUPPORT_MOVEONREBOOT
-      if (flags & DEL_REBOOT)
+      log_printf2("RMDir: RemoveDirectory invalid input(\"%s\")",buf);
+      g_exec_flags.exec_error++;
+    }
+    else if (file_exists(buf))
+    {
+      addtrailingslash(buf);
+      log_printf2("RMDir: RemoveDirectory(\"%s\")",buf);
+      SetFileAttributes(buf,FILE_ATTRIBUTE_NORMAL);
+      if (!RemoveDirectory(buf))
       {
-        log_printf2("RMDir: RemoveDirectory on Reboot(\"%s\")",buf);
-        update_status_text(LANG_DELETEONREBOOT,buf);
-        MoveFileOnReboot(buf,NULL);
+#ifdef NSIS_SUPPORT_MOVEONREBOOT
+        if (flags & DEL_REBOOT)
+        {
+          log_printf2("RMDir: RemoveDirectory on Reboot(\"%s\")",buf);
+          update_status_text(LANG_DELETEONREBOOT,buf);
+          MoveFileOnReboot(buf,NULL);
+        }
+        else
+#endif//NSIS_SUPPORT_MOVEONREBOOT
+        {
+          log_printf2("RMDir: RemoveDirectory failed(\"%s\")",buf);
+          g_exec_flags.exec_error++;
+        }
       }
       else
-#endif//NSIS_SUPPORT_MOVEONREBOOT
       {
-        log_printf2("RMDir: RemoveDirectory failed(\"%s\")",buf);
-        g_exec_flags.exec_error++;
+        update_status_text(LANG_REMOVEDIR,buf);
       }
     }
-    else
-      update_status_text(LANG_REMOVEDIR,buf);
   }
 #endif//NSIS_SUPPORT_RMDIR
 }
 
 char *NSISCALL addtrailingslash(char *str)
 {
-  if (lastchar(str)!='\\') lstrcat(str,"\\");
+  if (lastchar(str)!='\\') mystrcat(str,"\\");
   return str;
 }
 
@@ -284,6 +296,14 @@ int NSISCALL is_valid_instpath(char *s)
   if (!root)
     return 0;
 
+  // must be called after skip_root or AllowRootDirInstall won't work.
+  // validate_filename removes trailing blackslashes and so converts
+  // "C:\" to "C:" which is not a valid directory. skip_root returns
+  // NULL for "C:" so the above test returns 0.
+  // validate_filename is called so directories such as "C:\ " will
+  // not pass as a valid non-root directory.
+  validate_filename(root);
+
   if ((g_flags & CH_FLAGS_NO_ROOT_DIR) && (!*root || *root == '\\'))
     return 0;
 
@@ -334,20 +354,6 @@ void NSISCALL mini_memcpy(void *out, const void *in, int len)
   }
 }
 
-#ifdef __GNUC__
-void *memcpy(void *out, const void *in, size_t len)
-{
-  char *c_out=(char*)out;
-  char *c_in=(char *)in;
-  while (len-- > 0)
-  {
-    *c_out++=*c_in++;
-  }
-  return out;
-}
-#endif
-
-
 HANDLE NSISCALL myOpenFile(const char *fn, DWORD da, DWORD cd)
 {
   int attr = GetFileAttributes(fn);
@@ -381,16 +387,12 @@ char * NSISCALL my_GetTempFileName(char *buf, const char *dir)
 void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
 {
   BOOL fOk = 0;
-  HMODULE hLib=GetModuleHandle("KERNEL32.dll");
-  if (hLib)
+  typedef BOOL (WINAPI *mfea_t)(LPCSTR lpExistingFileName,LPCSTR lpNewFileName,DWORD dwFlags);
+  mfea_t mfea;
+  mfea=(mfea_t) myGetProcAddress("KERNEL32.dll","MoveFileExA");
+  if (mfea)
   {
-    typedef BOOL (WINAPI *mfea_t)(LPCSTR lpExistingFileName,LPCSTR lpNewFileName,DWORD dwFlags);
-    mfea_t mfea;
-    mfea=(mfea_t) GetProcAddress(hLib,"MoveFileExA");
-    if (mfea)
-    {
-      fOk=mfea(pszExisting, pszNew, MOVEFILE_DELAY_UNTIL_REBOOT|MOVEFILE_REPLACE_EXISTING);
-    }
+    fOk=mfea(pszExisting, pszNew, MOVEFILE_DELAY_UNTIL_REBOOT|MOVEFILE_REPLACE_EXISTING);
   }
 
   if (!fOk)
@@ -400,8 +402,11 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
     static char tmpbuf[1024];
     int cchRenameLine;
     char *szRenameSec = "[Rename]\r\n";
-    HANDLE hfile, hfilemap;
-    DWORD dwFileSize, dwRenameLinePos;
+    HANDLE hfile;
+    DWORD dwFileSize;
+    DWORD dwBytes;
+    DWORD dwRenameLinePos;
+    char *pszWinInit;
 
     int spn;
 
@@ -421,7 +426,7 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
     cchRenameLine = wsprintf(szRenameLine,"%s=%s\r\n",tmpbuf,wininit);
 
     GetWindowsDirectory(wininit, 1024-16);
-    lstrcat(wininit, "\\wininit.ini");
+    mystrcat(wininit, "\\wininit.ini");
     hfile = CreateFile(wininit,
         GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_ALWAYS,
         FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
@@ -429,13 +434,11 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
     if (hfile != INVALID_HANDLE_VALUE)
     {
       dwFileSize = GetFileSize(hfile, NULL);
-      hfilemap = CreateFileMapping(hfile, NULL, PAGE_READWRITE, 0, dwFileSize + cchRenameLine + 10, NULL);
+      pszWinInit = GlobalAlloc(GPTR, dwFileSize + cchRenameLine + 10);
 
-      if (hfilemap != NULL)
+      if (pszWinInit != NULL)
       {
-        LPSTR pszWinInit = (LPSTR) MapViewOfFile(hfilemap, FILE_MAP_WRITE, 0, 0, 0);
-
-        if (pszWinInit != NULL)
+        if (ReadFile(hfile, pszWinInit, dwFileSize, &dwBytes, NULL) && dwFileSize == dwBytes)
         {
           LPSTR pszRenameSecInFile = mystrstri(pszWinInit, szRenameSec);
           if (pszRenameSecInFile == NULL)
@@ -465,18 +468,16 @@ void NSISCALL MoveFileOnReboot(LPCTSTR pszExisting, LPCTSTR pszNew)
           mini_memcpy(&pszWinInit[dwRenameLinePos], szRenameLine, cchRenameLine);
           dwFileSize += cchRenameLine;
 
-          UnmapViewOfFile(pszWinInit);
+          SetFilePointer(hfile, 0, NULL, FILE_BEGIN);
+          WriteFile(hfile, pszWinInit, dwFileSize, &dwBytes, NULL);
 
-          //fOk++;
+          GlobalFree(pszWinInit);
         }
-        CloseHandle(hfilemap);
       }
-      SetFilePointer(hfile, dwFileSize, NULL, FILE_BEGIN);
-      SetEndOfFile(hfile);
+      
       CloseHandle(hfile);
     }
   }
-  //return fOk;
 
 #ifdef NSIS_SUPPORT_REBOOT
   g_exec_flags.exec_reboot++;
@@ -500,14 +501,15 @@ void NSISCALL myRegGetStr(HKEY root, const char *sub, const char *name, char *ou
 
 void NSISCALL myitoa(char *s, int d)
 {
-  wsprintf(s,"%d",d);
+  static const char c[] = "%d";
+  wsprintf(s,c,d);
 }
 
 int NSISCALL myatoi(char *s)
 {
   unsigned int v=0;
   int sign=1; // sign of positive
-  char m=10; // base of 0
+  char m=10; // base of 10
   char t='9'; // cap top of numbers at 9
 
   if (*s == '-')
@@ -548,7 +550,7 @@ int NSISCALL myatoi(char *s)
 // of a new function there should be about a couple of dozen or so calls.
 char * NSISCALL mystrcpy(char *out, const char *in)
 {
-  return lstrcpy(out, in);
+  return lstrcpyn(out, in, NSIS_MAX_STRLEN);
 }
 
 int NSISCALL mystrlen(const char *in)
@@ -556,6 +558,11 @@ int NSISCALL mystrlen(const char *in)
   return lstrlen(in);
 }
 
+char * NSISCALL mystrcat(char *out, const char *concat)
+{
+  return lstrcat(out, concat);
+}
+
 char ps_tmpbuf[NSIS_MAX_STRLEN*2];
 
 #define SYSREGKEY "Software\\Microsoft\\Windows\\CurrentVersion"
@@ -601,21 +608,21 @@ char * NSISCALL GetNSISString(char *outbuf, int strtab)
           append = "\\Microsoft\\Internet Explorer\\Quick Launch";
           x = 2;
         }
-        if (fldrs[0] == CSIDL_PROGRAM_FILES_COMMON)
+        else if (fldrs[0] == CSIDL_PROGRAM_FILES_COMMON)
         {
           myRegGetStr(HKEY_LOCAL_MACHINE, SYSREGKEY, "CommonFilesDir", out);
         }
-        if (fldrs[0] == CSIDL_PROGRAM_FILES)
+        else if (fldrs[0] == CSIDL_PROGRAM_FILES)
         {
           myRegGetStr(HKEY_LOCAL_MACHINE, SYSREGKEY, "ProgramFilesDir", out);
           if (!*out)
             mystrcpy(out, "C:\\Program Files");
         }
-        if (fldrs[0] == CSIDL_SYSTEM)
+        else if (fldrs[0] == CSIDL_SYSTEM)
         {
           GetSystemDirectory(out, NSIS_MAX_STRLEN);
         }
-        if (fldrs[0] == CSIDL_WINDOWS)
+        else if (fldrs[0] == CSIDL_WINDOWS)
         {
           GetWindowsDirectory(out, NSIS_MAX_STRLEN);
         }
@@ -644,11 +651,10 @@ char * NSISCALL GetNSISString(char *outbuf, int strtab)
 
         if (*out && append)
         {
-          lstrcat(out, append);
+          mystrcat(out, append);
         }
 
         validate_filename(out);
-        out += mystrlen(out);
       }
       else if (nVarIdx == NS_VAR_CODE)
       {
@@ -662,13 +668,12 @@ char * NSISCALL GetNSISString(char *outbuf, int strtab)
           // $LANGUAGE is just a number anyway...
           validate_filename(out);
         }
-        out += mystrlen(out);
       } // == VAR_CODES_START
       else if (nVarIdx == NS_LANG_CODE)
       {
         GetNSISString(out, -nData-1);
-        out += mystrlen(out);
       }
+      out += mystrlen(out);
     }
     else if (nVarIdx == NS_SKIP_CODE)
     {
@@ -681,15 +686,18 @@ char * NSISCALL GetNSISString(char *outbuf, int strtab)
   } // while
   *out = 0;
   if (outbuf)
-    return lstrcpyn(outbuf, ps_tmpbuf, NSIS_MAX_STRLEN);
+    return mystrcpy(outbuf, ps_tmpbuf);
   return ps_tmpbuf;
 }
 
-char * NSISCALL validate_filename(char *in) {
+void NSISCALL validate_filename(char *in) {
   char *nono = "*?|<>/\":";
   char *out;
   char *out_save;
-  while (*in == ' ') in = CharNext(in);
+
+  // ignoring spaces is wrong, " C:\blah" is invalid
+  //while (*in == ' ') in = CharNext(in);
+
   if (in[0] == '\\' && in[1] == '\\' && in[2] == '?' && in[3] == '\\')
   {
     // at least four bytes
@@ -719,14 +727,13 @@ char * NSISCALL validate_filename(char *in) {
     else
       break;
   } while (out_save < out);
-  return out_save;
 }
 
 #ifdef NSIS_CONFIG_LOG
 int log_dolog;
-char log_text[NSIS_MAX_STRLEN*4];
+char log_text[2048]; // 1024 for each wsprintf
 
-#ifndef NSIS_CONFIG_LOG_ODS
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
 void NSISCALL log_write(int close)
 {
   static HANDLE fp=INVALID_HANDLE_VALUE;
@@ -750,23 +757,94 @@ void NSISCALL log_write(int close)
     if (fp!=INVALID_HANDLE_VALUE)
     {
       DWORD d;
-      lstrcat(log_text,"\r\n");
+      mystrcat(log_text,"\r\n");
       WriteFile(fp,log_text,mystrlen(log_text),&d,NULL);
     }
   }
 }
-#endif//!NSIS_CONFIG_LOG_ODS
+#endif//!NSIS_CONFIG_LOG_ODS && !NSIS_CONFIG_LOG_STDOUT
+
+const char * _RegKeyHandleToName(HKEY hKey)
+{
+  if (hKey == HKEY_CLASSES_ROOT)
+    return "HKEY_CLASSES_ROOT";
+  else if (hKey == HKEY_CURRENT_USER)
+    return "HKEY_CURRENT_USER";
+  else if (hKey == HKEY_LOCAL_MACHINE)
+    return "HKEY_LOCAL_MACHINE";
+  else if (hKey == HKEY_USERS)
+    return "HKEY_USERS";
+  else if (hKey == HKEY_PERFORMANCE_DATA)
+    return "HKEY_PERFORMANCE_DATA";
+  else if (hKey == HKEY_CURRENT_CONFIG)
+    return "HKEY_CURRENT_CONFIG";
+  else if (hKey == HKEY_DYN_DATA)
+    return "HKEY_DYN_DATA";
+  else
+    return "invalid registry key";
+}
+
+void _LogData2Hex(char *buf, size_t buflen, unsigned char *data, size_t datalen)
+{
+  char *p = buf;
+
+  size_t i;
+
+  int dots = 0;
+  size_t bufbytes = buflen / 3; // 2 hex digits, one space/null
+
+  if (datalen > bufbytes)
+  {
+    bufbytes--;
+    dots = 1;
+  }
+  else
+    bufbytes = datalen;
+
+  for (i = 0; i < bufbytes; i++)
+  {
+    wsprintf(p, "%02x%c", data[i], (i == bufbytes - 1) ? '\0' : ' ');
+    p += 3;
+  }
+
+  if (dots)
+    mystrcat(buf, "...");
+}
+
+#ifdef NSIS_CONFIG_LOG_TIMESTAMP
+void log_timestamp(char *buf)
+{
+  SYSTEMTIME st;
+  GetLocalTime(&st);
+  wsprintf(buf,"[%04hu/%02hu/%02hu %02hu:%02hu:%02hu] ", st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
+}
+#else
+#  define log_timestamp(x)
+#endif//NSIS_CONFIG_LOG_TIMESTAMP
 
 void log_printf(char *format, ...)
 {
   va_list val;
   va_start(val,format);
-  wvsprintf(log_text,format,val);
+
+  log_text[0] = '\0';
+  log_timestamp(log_text);
+  wvsprintf(log_text+mystrlen(log_text),format,val);
+
   va_end(val);
 #ifdef NSIS_CONFIG_LOG_ODS
   if (log_dolog)
     OutputDebugString(log_text);
-#else
+#endif
+#ifdef NSIS_CONFIG_LOG_STDOUT
+  if (log_dolog && GetStdHandle(STD_OUTPUT_HANDLE) != INVALID_HANDLE_VALUE)
+  {
+    DWORD dwBytes;
+    WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), log_text, lstrlen(log_text), &dwBytes, NULL);
+    WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), "\n", 1, &dwBytes, NULL);
+  }
+#endif
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
   log_write(0);
 #endif
 }
@@ -787,3 +865,21 @@ WIN32_FIND_DATA * NSISCALL file_exists(char *buf)
   }
   return NULL;
 }
+
+void * NSISCALL myGetProcAddress(char *dll, char *func)
+{
+  HMODULE hModule = GetModuleHandle(dll);
+  if (!hModule)
+    hModule = LoadLibrary(dll);
+  if (!hModule)
+    return NULL;
+
+  return GetProcAddress(hModule, func);
+}
+
+void NSISCALL MessageLoop(UINT uCheckedMsg)
+{
+  MSG msg;
+  while (PeekMessage(&msg, NULL, uCheckedMsg, uCheckedMsg, PM_REMOVE))
+    DispatchMessage(&msg);
+}
